/********************************************************************************
** Form generated from reading UI file 'feature_selection_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FEATURE_SELECTION_DIALOG_H
#define UI_FEATURE_SELECTION_DIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_FeatureSelectionDialog
{
public:
    QDialogButtonBox *buttonBox;
    QLabel *feature_groups_label;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout;
    QRadioButton *capabilities_radioButton;
    QRadioButton *known_radioButton;
    QRadioButton *scan_radioButton;
    QRadioButton *mfg_radioButton;
    QRadioButton *color_radioButton;
    QHBoxLayout *horizontalLayout;
    QRadioButton *custom_radioButton;
    QLineEdit *custom_lineEdit;
    QLabel *featire_filters_label;
    QCheckBox *allCapabilities_checkbox;
    QCheckBox *onlyCapabilities_checkbox;
    QCheckBox *showUnsupported_checkbox;
    QCheckBox *includeTable_checkbox;

    void setupUi(QDialog *FeatureSelectionDialog)
    {
        if (FeatureSelectionDialog->objectName().isEmpty())
            FeatureSelectionDialog->setObjectName("FeatureSelectionDialog");
        FeatureSelectionDialog->resize(584, 443);
        buttonBox = new QDialogButtonBox(FeatureSelectionDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setGeometry(QRect(30, 370, 341, 61));
        QFont font;
        font.setPointSize(10);
        buttonBox->setFont(font);
        buttonBox->setFocusPolicy(Qt::StrongFocus);
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Help|QDialogButtonBox::Ok|QDialogButtonBox::Reset);
        buttonBox->setCenterButtons(true);
        feature_groups_label = new QLabel(FeatureSelectionDialog);
        feature_groups_label->setObjectName("feature_groups_label");
        feature_groups_label->setGeometry(QRect(30, 20, 171, 31));
        QFont font1;
        font1.setPointSize(14);
        feature_groups_label->setFont(font1);
        layoutWidget = new QWidget(FeatureSelectionDialog);
        layoutWidget->setObjectName("layoutWidget");
        layoutWidget->setGeometry(QRect(40, 50, 511, 166));
        verticalLayout = new QVBoxLayout(layoutWidget);
        verticalLayout->setSpacing(3);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        capabilities_radioButton = new QRadioButton(layoutWidget);
        capabilities_radioButton->setObjectName("capabilities_radioButton");
        capabilities_radioButton->setEnabled(true);
        QFont font2;
        font2.setPointSize(10);
        font2.setBold(false);
        font2.setItalic(false);
        capabilities_radioButton->setFont(font2);
        capabilities_radioButton->setCheckable(true);

        verticalLayout->addWidget(capabilities_radioButton);

        known_radioButton = new QRadioButton(layoutWidget);
        known_radioButton->setObjectName("known_radioButton");
        known_radioButton->setMinimumSize(QSize(0, 23));
        known_radioButton->setFont(font2);

        verticalLayout->addWidget(known_radioButton);

        scan_radioButton = new QRadioButton(layoutWidget);
        scan_radioButton->setObjectName("scan_radioButton");
        scan_radioButton->setFont(font);

        verticalLayout->addWidget(scan_radioButton);

        mfg_radioButton = new QRadioButton(layoutWidget);
        mfg_radioButton->setObjectName("mfg_radioButton");
        mfg_radioButton->setFont(font);

        verticalLayout->addWidget(mfg_radioButton);

        color_radioButton = new QRadioButton(layoutWidget);
        color_radioButton->setObjectName("color_radioButton");
        color_radioButton->setFont(font);

        verticalLayout->addWidget(color_radioButton);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        custom_radioButton = new QRadioButton(layoutWidget);
        custom_radioButton->setObjectName("custom_radioButton");
        custom_radioButton->setFont(font);

        horizontalLayout->addWidget(custom_radioButton);

        custom_lineEdit = new QLineEdit(layoutWidget);
        custom_lineEdit->setObjectName("custom_lineEdit");

        horizontalLayout->addWidget(custom_lineEdit);


        verticalLayout->addLayout(horizontalLayout);

        featire_filters_label = new QLabel(FeatureSelectionDialog);
        featire_filters_label->setObjectName("featire_filters_label");
        featire_filters_label->setGeometry(QRect(30, 220, 171, 31));
        featire_filters_label->setFont(font1);
        allCapabilities_checkbox = new QCheckBox(FeatureSelectionDialog);
        allCapabilities_checkbox->setObjectName("allCapabilities_checkbox");
        allCapabilities_checkbox->setGeometry(QRect(40, 260, 313, 22));
        allCapabilities_checkbox->setFont(font);
        onlyCapabilities_checkbox = new QCheckBox(FeatureSelectionDialog);
        onlyCapabilities_checkbox->setObjectName("onlyCapabilities_checkbox");
        onlyCapabilities_checkbox->setGeometry(QRect(40, 290, 313, 23));
        onlyCapabilities_checkbox->setFont(font);
        showUnsupported_checkbox = new QCheckBox(FeatureSelectionDialog);
        showUnsupported_checkbox->setObjectName("showUnsupported_checkbox");
        showUnsupported_checkbox->setEnabled(true);
        showUnsupported_checkbox->setGeometry(QRect(40, 320, 313, 20));
        QFont font3;
        font3.setPointSize(10);
        font3.setStrikeOut(false);
        showUnsupported_checkbox->setFont(font3);
        showUnsupported_checkbox->setCheckable(true);
        includeTable_checkbox = new QCheckBox(FeatureSelectionDialog);
        includeTable_checkbox->setObjectName("includeTable_checkbox");
        includeTable_checkbox->setEnabled(false);
        includeTable_checkbox->setGeometry(QRect(40, 350, 313, 23));
        includeTable_checkbox->setFont(font3);
        includeTable_checkbox->setCheckable(false);

        retranslateUi(FeatureSelectionDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, FeatureSelectionDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(FeatureSelectionDialog);
    } // setupUi

    void retranslateUi(QDialog *FeatureSelectionDialog)
    {
        FeatureSelectionDialog->setWindowTitle(QCoreApplication::translate("FeatureSelectionDialog", "ddcui - Feature Selection", nullptr));
        feature_groups_label->setText(QCoreApplication::translate("FeatureSelectionDialog", "<html><head/><body><p><span style=\" font-size:12pt;\">Feature Groups</span></p></body></html>", nullptr));
#if QT_CONFIG(whatsthis)
        capabilities_radioButton->setWhatsThis(QCoreApplication::translate("FeatureSelectionDialog", "<html><head/><body><p>This is whats this text</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        capabilities_radioButton->setText(QCoreApplication::translate("FeatureSelectionDialog", "CAPABILITIES - Features specified in the capabilities string", nullptr));
#if QT_CONFIG(tooltip)
        known_radioButton->setToolTip(QCoreApplication::translate("FeatureSelectionDialog", "<html><head/><body><p>This is tooltip text</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        known_radioButton->setText(QCoreApplication::translate("FeatureSelectionDialog", "&MCCS - Features defined in the Monitor Control Command Set", nullptr));
        scan_radioButton->setText(QCoreApplication::translate("FeatureSelectionDialog", "SCAN - All possible feature codes", nullptr));
        mfg_radioButton->setText(QCoreApplication::translate("FeatureSelectionDialog", "&MANUFACTURER - Manufacturer specific features", nullptr));
        color_radioButton->setText(QCoreApplication::translate("FeatureSelectionDialog", "COLOR - Color related features", nullptr));
        custom_radioButton->setText(QCoreApplication::translate("FeatureSelectionDialog", "CUSTOM  ", nullptr));
        featire_filters_label->setText(QCoreApplication::translate("FeatureSelectionDialog", "<html><head/><body><p><span style=\" font-size:12pt;\">Feature Filters</span></p></body></html>", nullptr));
        allCapabilities_checkbox->setText(QCoreApplication::translate("FeatureSelectionDialog", "Include all features in the capabilities string", nullptr));
        onlyCapabilities_checkbox->setText(QCoreApplication::translate("FeatureSelectionDialog", "Include only features in the capabilities string", nullptr));
        showUnsupported_checkbox->setText(QCoreApplication::translate("FeatureSelectionDialog", "Show unsupported features", nullptr));
        includeTable_checkbox->setText(QCoreApplication::translate("FeatureSelectionDialog", "Include Table type features", nullptr));
    } // retranslateUi

};

namespace Ui {
    class FeatureSelectionDialog: public Ui_FeatureSelectionDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FEATURE_SELECTION_DIALOG_H
