/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../ddcui-0.6.0/src/main/mainwindow.h"
#include <QtGui/qtextcursor.h>
#include <QtGui/qscreen.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10MainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto MainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN10MainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainWindow",
        "featureSelectionChanged",
        "",
        "signalMonitorSummaryView",
        "signalCapabilitiesView",
        "signalFeaturesView",
        "userInterfaceOptionsChanged",
        "showStats",
        "resetStats",
        "reportApplicationEventLoopStarted",
        "signalControlKeyPressed",
        "onoff",
        "longRunningTaskStart",
        "longRunningTaskEnd",
        "setTransitoryStatusMsg",
        "msg",
        "showSerialMsgBox",
        "title",
        "text",
        "QMessageBox::Icon",
        "icon",
        "forControlKeyRequired_changed",
        "on_actionRedetect_triggered",
        "forDisplayChanged",
        "DDCA_Display_Status_Event",
        "evt",
        "keyPressEvent",
        "QKeyEvent*",
        "ev",
        "keyReleaseEvent",
        "on_actionMonitorSummary_triggered",
        "on_actionCapabilities_triggered",
        "on_actionFeaturesScrollArea_triggered",
        "on_actionRescan_triggered",
        "on_actionDebugActionsDialog_triggered",
        "for_resetStats_triggered",
        "for_reportStats_triggered",
        "DDCA_Stats_Type",
        "stats_type",
        "showTheadData",
        "on_actionDebugLocks_triggered",
        "on_actionFeatureSelectionDialog_triggered",
        "for_actionFeatureSelectionDialog_accepted",
        "on_actionOtherOptionsDialog_triggered",
        "on_actionUserInterfaceOptionsDialog_triggered",
        "for_actionUserInterfaceOptionsDialog_accept",
        "on_actionContentsHelp_triggered",
        "on_actionAbout_triggered",
        "on_actionAbout_Qt_triggered",
        "displaySelectorCombobox_currentIndexChanged",
        "index",
        "quitShortcut"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'featureSelectionChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'signalMonitorSummaryView'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'signalCapabilitiesView'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'signalFeaturesView'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'userInterfaceOptionsChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showStats'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resetStats'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'reportApplicationEventLoopStarted'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'signalControlKeyPressed'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'longRunningTaskStart'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'longRunningTaskEnd'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setTransitoryStatusMsg'
        QtMocHelpers::SlotData<void(QString)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'showSerialMsgBox'
        QtMocHelpers::SlotData<void(QString, QString, QMessageBox::Icon)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { QMetaType::QString, 18 }, { 0x80000000 | 19, 20 },
        }}),
        // Slot 'forControlKeyRequired_changed'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'on_actionRedetect_triggered'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'forDisplayChanged'
        QtMocHelpers::SlotData<void(DDCA_Display_Status_Event)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'keyPressEvent'
        QtMocHelpers::SlotData<void(QKeyEvent *)>(26, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'keyReleaseEvent'
        QtMocHelpers::SlotData<void(QKeyEvent *)>(29, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'on_actionMonitorSummary_triggered'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionCapabilities_triggered'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionFeaturesScrollArea_triggered'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionRescan_triggered'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionDebugActionsDialog_triggered'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'for_resetStats_triggered'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'for_reportStats_triggered'
        QtMocHelpers::SlotData<void(DDCA_Stats_Type, bool)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 37, 38 }, { QMetaType::Bool, 39 },
        }}),
        // Slot 'on_actionDebugLocks_triggered'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionFeatureSelectionDialog_triggered'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'for_actionFeatureSelectionDialog_accepted'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionOtherOptionsDialog_triggered'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionUserInterfaceOptionsDialog_triggered'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'for_actionUserInterfaceOptionsDialog_accept'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionContentsHelp_triggered'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionAbout_triggered'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionAbout_Qt_triggered'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'displaySelectorCombobox_currentIndexChanged'
        QtMocHelpers::SlotData<void(int)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 50 },
        }}),
        // Slot 'quitShortcut'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWindow, qt_meta_tag_ZN10MainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10MainWindowE_t>.metaTypes,
    nullptr
} };

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->featureSelectionChanged(); break;
        case 1: _t->signalMonitorSummaryView(); break;
        case 2: _t->signalCapabilitiesView(); break;
        case 3: _t->signalFeaturesView(); break;
        case 4: _t->userInterfaceOptionsChanged(); break;
        case 5: _t->showStats(); break;
        case 6: _t->resetStats(); break;
        case 7: _t->reportApplicationEventLoopStarted(); break;
        case 8: _t->signalControlKeyPressed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->longRunningTaskStart(); break;
        case 10: _t->longRunningTaskEnd(); break;
        case 11: _t->setTransitoryStatusMsg((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->showSerialMsgBox((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QMessageBox::Icon>>(_a[3]))); break;
        case 13: _t->forControlKeyRequired_changed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->on_actionRedetect_triggered(); break;
        case 15: _t->forDisplayChanged((*reinterpret_cast<std::add_pointer_t<DDCA_Display_Status_Event>>(_a[1]))); break;
        case 16: _t->keyPressEvent((*reinterpret_cast<std::add_pointer_t<QKeyEvent*>>(_a[1]))); break;
        case 17: _t->keyReleaseEvent((*reinterpret_cast<std::add_pointer_t<QKeyEvent*>>(_a[1]))); break;
        case 18: _t->on_actionMonitorSummary_triggered(); break;
        case 19: _t->on_actionCapabilities_triggered(); break;
        case 20: _t->on_actionFeaturesScrollArea_triggered(); break;
        case 21: _t->on_actionRescan_triggered(); break;
        case 22: _t->on_actionDebugActionsDialog_triggered(); break;
        case 23: _t->for_resetStats_triggered(); break;
        case 24: _t->for_reportStats_triggered((*reinterpret_cast<std::add_pointer_t<DDCA_Stats_Type>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 25: _t->on_actionDebugLocks_triggered(); break;
        case 26: _t->on_actionFeatureSelectionDialog_triggered(); break;
        case 27: _t->for_actionFeatureSelectionDialog_accepted(); break;
        case 28: _t->on_actionOtherOptionsDialog_triggered(); break;
        case 29: _t->on_actionUserInterfaceOptionsDialog_triggered(); break;
        case 30: _t->for_actionUserInterfaceOptionsDialog_accept(); break;
        case 31: _t->on_actionContentsHelp_triggered(); break;
        case 32: _t->on_actionAbout_triggered(); break;
        case 33: _t->on_actionAbout_Qt_triggered(); break;
        case 34: _t->displaySelectorCombobox_currentIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 35: _t->quitShortcut(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::featureSelectionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::signalMonitorSummaryView, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::signalCapabilitiesView, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::signalFeaturesView, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::userInterfaceOptionsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::showStats, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::resetStats, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::reportApplicationEventLoopStarted, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)(bool )>(_a, &MainWindow::signalControlKeyPressed, 8))
            return;
    }
}

const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 36;
    }
    return _id;
}

// SIGNAL 0
void MainWindow::featureSelectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MainWindow::signalMonitorSummaryView()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MainWindow::signalCapabilitiesView()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MainWindow::signalFeaturesView()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MainWindow::userInterfaceOptionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void MainWindow::showStats()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void MainWindow::resetStats()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void MainWindow::reportApplicationEventLoopStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void MainWindow::signalControlKeyPressed(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
