/****************************************************************************
** Meta object code from reading C++ file 'feature_base_model.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../ddcui-0.6.0/src/nongui/feature_base_model.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'feature_base_model.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16FeatureBaseModelE_t {};
} // unnamed namespace

template <> constexpr inline auto FeatureBaseModel::qt_create_metaobjectdata<qt_meta_tag_ZN16FeatureBaseModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FeatureBaseModel",
        "signalStartInitialLoad",
        "",
        "signalEndInitialLoad",
        "signalFeatureAdded",
        "FeatureValue",
        "fv",
        "signalFeatureUpdated3",
        "const char*",
        "caller",
        "uint8_t",
        "feature_code",
        "sh",
        "sl",
        "signalVcpRequest",
        "VcpRequest*",
        "rqst",
        "signalModelError",
        "featureCode",
        "msg",
        "signalStatusMsg"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalStartInitialLoad'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'signalEndInitialLoad'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'signalFeatureAdded'
        QtMocHelpers::SignalData<void(FeatureValue)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'signalFeatureUpdated3'
        QtMocHelpers::SignalData<void(const char *, uint8_t, uint8_t, uint8_t)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 10, 11 }, { 0x80000000 | 10, 12 }, { 0x80000000 | 10, 13 },
        }}),
        // Signal 'signalVcpRequest'
        QtMocHelpers::SignalData<void(VcpRequest *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'signalModelError'
        QtMocHelpers::SignalData<void(uint8_t, QString)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 18 }, { QMetaType::QString, 19 },
        }}),
        // Signal 'signalStatusMsg'
        QtMocHelpers::SignalData<void(QString)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FeatureBaseModel, qt_meta_tag_ZN16FeatureBaseModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FeatureBaseModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16FeatureBaseModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16FeatureBaseModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16FeatureBaseModelE_t>.metaTypes,
    nullptr
} };

void FeatureBaseModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FeatureBaseModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalStartInitialLoad(); break;
        case 1: _t->signalEndInitialLoad(); break;
        case 2: _t->signalFeatureAdded((*reinterpret_cast<std::add_pointer_t<FeatureValue>>(_a[1]))); break;
        case 3: _t->signalFeatureUpdated3((*reinterpret_cast<std::add_pointer_t<const char*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint8_t>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint8_t>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint8_t>>(_a[4]))); break;
        case 4: _t->signalVcpRequest((*reinterpret_cast<std::add_pointer_t<VcpRequest*>>(_a[1]))); break;
        case 5: _t->signalModelError((*reinterpret_cast<std::add_pointer_t<uint8_t>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->signalStatusMsg((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< FeatureValue >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FeatureBaseModel::*)()>(_a, &FeatureBaseModel::signalStartInitialLoad, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FeatureBaseModel::*)()>(_a, &FeatureBaseModel::signalEndInitialLoad, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FeatureBaseModel::*)(FeatureValue )>(_a, &FeatureBaseModel::signalFeatureAdded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FeatureBaseModel::*)(const char * , uint8_t , uint8_t , uint8_t )>(_a, &FeatureBaseModel::signalFeatureUpdated3, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (FeatureBaseModel::*)(VcpRequest * )>(_a, &FeatureBaseModel::signalVcpRequest, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (FeatureBaseModel::*)(uint8_t , QString )>(_a, &FeatureBaseModel::signalModelError, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (FeatureBaseModel::*)(QString )>(_a, &FeatureBaseModel::signalStatusMsg, 6))
            return;
    }
}

const QMetaObject *FeatureBaseModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FeatureBaseModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16FeatureBaseModelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int FeatureBaseModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void FeatureBaseModel::signalStartInitialLoad()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void FeatureBaseModel::signalEndInitialLoad()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void FeatureBaseModel::signalFeatureAdded(FeatureValue _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void FeatureBaseModel::signalFeatureUpdated3(const char * _t1, uint8_t _t2, uint8_t _t3, uint8_t _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 4
void FeatureBaseModel::signalVcpRequest(VcpRequest * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void FeatureBaseModel::signalModelError(uint8_t _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void FeatureBaseModel::signalStatusMsg(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
