/********************************************************************************
** Form generated from reading UI file 'debug_actions_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DEBUG_ACTIONS_DIALOG_H
#define UI_DEBUG_ACTIONS_DIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>

QT_BEGIN_NAMESPACE

class Ui_DebugActionsDialog
{
public:
    QDialogButtonBox *actionButtonBox;
    QLabel *title;
    QGroupBox *dumpStatsGroupBox;
    QPushButton *all_pushButton;
    QPushButton *calls_pushButton;
    QPushButton *tries_pushButton;
    QPushButton *errors_pushButton;
    QPushButton *elapsed_pushButton;
    QCheckBox *extendedStats_checkBox;
    QPushButton *api_pushButton;
    QPushButton *resetStats_pushButton;

    void setupUi(QDialog *DebugActionsDialog)
    {
        if (DebugActionsDialog->objectName().isEmpty())
            DebugActionsDialog->setObjectName("DebugActionsDialog");
        DebugActionsDialog->resize(235, 458);
        actionButtonBox = new QDialogButtonBox(DebugActionsDialog);
        actionButtonBox->setObjectName("actionButtonBox");
        actionButtonBox->setGeometry(QRect(10, 410, 191, 32));
        actionButtonBox->setOrientation(Qt::Horizontal);
        actionButtonBox->setStandardButtons(QDialogButtonBox::Help|QDialogButtonBox::Ok);
        actionButtonBox->setCenterButtons(true);
        title = new QLabel(DebugActionsDialog);
        title->setObjectName("title");
        title->setGeometry(QRect(20, 10, 241, 31));
        QFont font;
        font.setPointSize(14);
        title->setFont(font);
        dumpStatsGroupBox = new QGroupBox(DebugActionsDialog);
        dumpStatsGroupBox->setObjectName("dumpStatsGroupBox");
        dumpStatsGroupBox->setGeometry(QRect(20, 40, 171, 311));
        all_pushButton = new QPushButton(dumpStatsGroupBox);
        all_pushButton->setObjectName("all_pushButton");
        all_pushButton->setGeometry(QRect(40, 30, 88, 34));
        calls_pushButton = new QPushButton(dumpStatsGroupBox);
        calls_pushButton->setObjectName("calls_pushButton");
        calls_pushButton->setGeometry(QRect(40, 70, 88, 34));
        tries_pushButton = new QPushButton(dumpStatsGroupBox);
        tries_pushButton->setObjectName("tries_pushButton");
        tries_pushButton->setGeometry(QRect(40, 150, 88, 34));
        errors_pushButton = new QPushButton(dumpStatsGroupBox);
        errors_pushButton->setObjectName("errors_pushButton");
        errors_pushButton->setGeometry(QRect(40, 110, 88, 34));
        elapsed_pushButton = new QPushButton(dumpStatsGroupBox);
        elapsed_pushButton->setObjectName("elapsed_pushButton");
        elapsed_pushButton->setGeometry(QRect(40, 190, 88, 34));
        extendedStats_checkBox = new QCheckBox(dumpStatsGroupBox);
        extendedStats_checkBox->setObjectName("extendedStats_checkBox");
        extendedStats_checkBox->setGeometry(QRect(0, 270, 161, 22));
        api_pushButton = new QPushButton(dumpStatsGroupBox);
        api_pushButton->setObjectName("api_pushButton");
        api_pushButton->setGeometry(QRect(40, 230, 88, 34));
        resetStats_pushButton = new QPushButton(DebugActionsDialog);
        resetStats_pushButton->setObjectName("resetStats_pushButton");
        resetStats_pushButton->setGeometry(QRect(50, 360, 121, 34));

        retranslateUi(DebugActionsDialog);
        QObject::connect(actionButtonBox, &QDialogButtonBox::accepted, DebugActionsDialog, qOverload<>(&QDialog::accept));

        QMetaObject::connectSlotsByName(DebugActionsDialog);
    } // setupUi

    void retranslateUi(QDialog *DebugActionsDialog)
    {
        DebugActionsDialog->setWindowTitle(QCoreApplication::translate("DebugActionsDialog", "Dialog", nullptr));
        title->setText(QCoreApplication::translate("DebugActionsDialog", "Execution Statistics", nullptr));
        dumpStatsGroupBox->setTitle(QCoreApplication::translate("DebugActionsDialog", "Dump Statistics", nullptr));
        all_pushButton->setText(QCoreApplication::translate("DebugActionsDialog", "All", nullptr));
        calls_pushButton->setText(QCoreApplication::translate("DebugActionsDialog", "Calls", nullptr));
        tries_pushButton->setText(QCoreApplication::translate("DebugActionsDialog", "Tries", nullptr));
        errors_pushButton->setText(QCoreApplication::translate("DebugActionsDialog", "Errors", nullptr));
        elapsed_pushButton->setText(QCoreApplication::translate("DebugActionsDialog", "Elapsed", nullptr));
        extendedStats_checkBox->setText(QCoreApplication::translate("DebugActionsDialog", "Extended Stats", nullptr));
        api_pushButton->setText(QCoreApplication::translate("DebugActionsDialog", "API", nullptr));
        resetStats_pushButton->setText(QCoreApplication::translate("DebugActionsDialog", "Reset Stats", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DebugActionsDialog: public Ui_DebugActionsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DEBUG_ACTIONS_DIALOG_H
