/********************************************************************************
** Form generated from reading UI file 'user_interface_options_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_USER_INTERFACE_OPTIONS_DIALOG_H
#define UI_USER_INTERFACE_OPTIONS_DIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>

QT_BEGIN_NAMESPACE

class Ui_UserInterfaceOptionsDialog
{
public:
    QDialogButtonBox *actionButtonBox;
    QCheckBox *ckrCheckBox;

    void setupUi(QDialog *UserInterfaceOptionsDialog)
    {
        if (UserInterfaceOptionsDialog->objectName().isEmpty())
            UserInterfaceOptionsDialog->setObjectName("UserInterfaceOptionsDialog");
        UserInterfaceOptionsDialog->setWindowModality(Qt::ApplicationModal);
        UserInterfaceOptionsDialog->resize(400, 108);
        UserInterfaceOptionsDialog->setModal(true);
        actionButtonBox = new QDialogButtonBox(UserInterfaceOptionsDialog);
        actionButtonBox->setObjectName("actionButtonBox");
        actionButtonBox->setGeometry(QRect(10, 50, 341, 32));
        QFont font;
        font.setPointSize(10);
        actionButtonBox->setFont(font);
        actionButtonBox->setFocusPolicy(Qt::StrongFocus);
        actionButtonBox->setOrientation(Qt::Horizontal);
        actionButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Help|QDialogButtonBox::Ok|QDialogButtonBox::Reset);
        actionButtonBox->setCenterButtons(true);
        ckrCheckBox = new QCheckBox(UserInterfaceOptionsDialog);
        ckrCheckBox->setObjectName("ckrCheckBox");
        ckrCheckBox->setGeometry(QRect(20, 20, 311, 22));

        retranslateUi(UserInterfaceOptionsDialog);
        QObject::connect(actionButtonBox, &QDialogButtonBox::accepted, UserInterfaceOptionsDialog, qOverload<>(&QDialog::accept));
        QObject::connect(actionButtonBox, &QDialogButtonBox::rejected, UserInterfaceOptionsDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(UserInterfaceOptionsDialog);
    } // setupUi

    void retranslateUi(QDialog *UserInterfaceOptionsDialog)
    {
        UserInterfaceOptionsDialog->setWindowTitle(QCoreApplication::translate("UserInterfaceOptionsDialog", "Dialog", nullptr));
        ckrCheckBox->setText(QCoreApplication::translate("UserInterfaceOptionsDialog", "Require control key to change values", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UserInterfaceOptionsDialog: public Ui_UserInterfaceOptionsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_USER_INTERFACE_OPTIONS_DIALOG_H
