// ddcui_parms.cpp

// Copyright (C) 2019-2020 Sanford Rockowitz <rockowitz@minsoft.com>
// SPDX-License-Identifier: GPL-2.0-or-later

#include "ddcui_parms.h"

const bool debugSignals            = false;
const bool debugValueWidgetSignals = false;
const bool debugFeatureLists       = false;
const bool debugFeatureSelection   = false;
const bool debugLayout             = false;
const bool debugNcValues           = false;
const bool debugFeatureDimensions  = false;
