/********************************************************************************
** Form generated from reading UI file 'nc_values_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NC_VALUES_DIALOG_H
#define UI_NC_VALUES_DIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>

QT_BEGIN_NAMESPACE

class Ui_NcValuesDialog
{
public:
    QDialogButtonBox *buttonBox;
    QRadioButton *mccsNcValuesButton;
    QRadioButton *capabilitiesNcValuesButton;
    QLabel *nc_value_options_header;
    QRadioButton *bothNcValuesButton;
    QButtonGroup *ncValueSourceButtonGroup;

    void setupUi(QDialog *NcValuesDialog)
    {
        if (NcValuesDialog->objectName().isEmpty())
            NcValuesDialog->setObjectName("NcValuesDialog");
        NcValuesDialog->setWindowModality(Qt::ApplicationModal);
        NcValuesDialog->resize(477, 165);
        NcValuesDialog->setContextMenuPolicy(Qt::NoContextMenu);
        NcValuesDialog->setModal(true);
        buttonBox = new QDialogButtonBox(NcValuesDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setGeometry(QRect(20, 110, 341, 32));
        QFont font;
        font.setPointSize(10);
        buttonBox->setFont(font);
        buttonBox->setFocusPolicy(Qt::StrongFocus);
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Help|QDialogButtonBox::Ok|QDialogButtonBox::Reset);
        buttonBox->setCenterButtons(false);
        mccsNcValuesButton = new QRadioButton(NcValuesDialog);
        ncValueSourceButtonGroup = new QButtonGroup(NcValuesDialog);
        ncValueSourceButtonGroup->setObjectName("ncValueSourceButtonGroup");
        ncValueSourceButtonGroup->addButton(mccsNcValuesButton);
        mccsNcValuesButton->setObjectName("mccsNcValuesButton");
        mccsNcValuesButton->setGeometry(QRect(20, 30, 311, 22));
        mccsNcValuesButton->setFont(font);
        capabilitiesNcValuesButton = new QRadioButton(NcValuesDialog);
        ncValueSourceButtonGroup->addButton(capabilitiesNcValuesButton);
        capabilitiesNcValuesButton->setObjectName("capabilitiesNcValuesButton");
        capabilitiesNcValuesButton->setGeometry(QRect(20, 50, 291, 22));
        capabilitiesNcValuesButton->setFont(font);
        nc_value_options_header = new QLabel(NcValuesDialog);
        nc_value_options_header->setObjectName("nc_value_options_header");
        nc_value_options_header->setGeometry(QRect(20, 10, 441, 18));
        bothNcValuesButton = new QRadioButton(NcValuesDialog);
        ncValueSourceButtonGroup->addButton(bothNcValuesButton);
        bothNcValuesButton->setObjectName("bothNcValuesButton");
        bothNcValuesButton->setGeometry(QRect(20, 70, 261, 22));
        bothNcValuesButton->setFont(font);

        retranslateUi(NcValuesDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, NcValuesDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, NcValuesDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(NcValuesDialog);
    } // setupUi

    void retranslateUi(QDialog *NcValuesDialog)
    {
        NcValuesDialog->setWindowTitle(QCoreApplication::translate("NcValuesDialog", "Dialog", nullptr));
#if QT_CONFIG(tooltip)
        mccsNcValuesButton->setToolTip(QCoreApplication::translate("NcValuesDialog", "List of valid values are taken from the MCCS specification", nullptr));
#endif // QT_CONFIG(tooltip)
        mccsNcValuesButton->setText(QCoreApplication::translate("NcValuesDialog", "&Monitor Control Command Set", nullptr));
#if QT_CONFIG(tooltip)
        capabilitiesNcValuesButton->setToolTip(QCoreApplication::translate("NcValuesDialog", "List of valid values taken from the capabilities string, names of valid values taken from MCCS spec", nullptr));
#endif // QT_CONFIG(tooltip)
        capabilitiesNcValuesButton->setText(QCoreApplication::translate("NcValuesDialog", "&Capabilities String", nullptr));
        nc_value_options_header->setText(QCoreApplication::translate("NcValuesDialog", "Source of Non-Continuous Feature Values:", nullptr));
#if QT_CONFIG(tooltip)
        bothNcValuesButton->setToolTip(QCoreApplication::translate("NcValuesDialog", "Valid values are those either in the MCCS spec or the capabilities string", nullptr));
#endif // QT_CONFIG(tooltip)
        bothNcValuesButton->setText(QCoreApplication::translate("NcValuesDialog", "Both &MCCS and Capabilities", nullptr));
    } // retranslateUi

};

namespace Ui {
    class NcValuesDialog: public Ui_NcValuesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_NC_VALUES_DIALOG_H
